function dsi_hycom(varargin)
% updates the DataSetInventory structures
% without arguments for all the following DataSetBranches
% url location of the dataset files

if nargin < 1

List.DataSetBranch={
%'HYCOM_Global_Near_Real_Time_Outputs';
'HYCOM_Global_Hindcast';
'HYCOM_Global_Simulation';
'HYCOM_Atlantic_Ocean_Prediction_System';    
};

    for k=1:length(List.DataSetBranch)
%    disp(['DSI_' List.DataSetBranch{k} ])
    eval(['DSI_' List.DataSetBranch{k} ';'])
    end

if ~exist('dsi_hycom.mat','file')
    save dsi_hycom.mat List
    else
    save dsi_hycom.mat -append List
    end

elseif nargin > 1
disp('Incorrect number of arguments.'); return
else
% with one argument for the specified DataSetBranch only
eval(varargin{1});
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function DSI=DSI_HYCOM_Global_Near_Real_Time_Outputs
% create DataSetInventory of HYCOM

% Example of loaddap argument
% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_nrt_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
DSI.DataSetBranch='HYCOM_Global_Near_Real_Time_Outputs';

% a general request employed by get_DataSet program has the following pattern
% loaddap([URLSITE URLPATH URLFILE '?' URLCVAR URLCTIME URLCLAT URLCLON URLCDEPTH])
%   constraints URLCTIME URLCLAT URLCLON URLCDEPTH can be in different order
%   constraints URLCLAT URLCLON URLCDEPTH will be generated by program get

% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_nrt_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
DSI.URLSITE='http://hycom.coaps.fsu.edu';
DSI.URLPATH='/thredds/dodsC/';
DSI.URLFILE='glb_nrt_analysis';
URL=[DSI.URLSITE DSI.URLPATH DSI.URLFILE];

% A=loaddap('-A','http://hycom.coaps.fsu.edu/thredds/dodsC/glb_nrt_analysis')
%A = 
%                                   Y: [1x1 struct]
%                                   X: [1x1 struct]
%                            Latitude: [1x1 struct]
%                           Longitude: [1x1 struct]
%                                  MT: [1x1 struct]
%                                Date: [1x1 struct]
%                                qtot: [1x1 struct]
%                                 emp: [1x1 struct]
%           surface_temperature_trend: [1x1 struct]
%              surface_salinity_trend: [1x1 struct]
%                                 ssh: [1x1 struct]
%    surface_boundary_layer_thickness: [1x1 struct]
%               mixed_layer_thickness: [1x1 struct]
%                               Depth: [1x1 struct]
%                                   u: [1x1 struct]
%                                   v: [1x1 struct]
%                         temperature: [1x1 struct]
%                            salinity: [1x1 struct]
%                   Global_Attributes: [1x1 struct]

% define list of fields or variables available in the dataset
LL={'latitude','longitude'};
LLD={'latitude','longitude','depth'};
TYX={'MT','Y','X'};
TDYX={'MT','Depth','Y','X'};

DSI.Fields={
%    opendap                      dataset                            menu                        opendap   dataset
%    name                       , name                             , name                       ,dim,coord,dim,coord,CBNum
    'downward_heat_flux_in_air' ,'qtot'                            ,'downward_heat_flux_in_air (qtot)' ,2  ,LL   ,3  ,TYX ,16;
    'water_flux_into_ocean'     ,'emp'                             ,'water_flux_into_ocean (emp)'      ,2  ,LL   ,3  ,TYX ,17;
    'surface_temperature_trend' ,'surface_temperature_trend'       ,'temperature_trend'                ,2  ,LL   ,3  ,TYX ,13;
    'surface_salinity_trend'    ,'surface_salinity_trend'          ,'salinity_trend'                   ,2  ,LL   ,3  ,TYX ,12;
    'sea_surface_elevation'     ,'ssh'                             ,'sea_surface_elevation (ssh)'      ,2  ,LL   ,3  ,TYX ,14;
%   'mixed_layer_u_velocity'    ,'mixed_layer_u_velocity'          ,'u_velocity'                       ,2  ,LL   ,3  ,TYX ,10;
%   'mixed_layer_v_velocity'    ,'mixed_layer_v_velocity'          ,'v_velocity'                       ,2  ,LL   ,3  ,TYX ,11;
    'boundary_layer_thickness'  ,'surface_boundary_layer_thickness','boundary_layer_thickness'         ,2  ,LL   ,3  ,TYX ,15;
    'thickness'                 ,'mixed_layer_thickness'           ,'thickness'                        ,2  ,LL   ,3  ,TYX , 9;
%   'mixed_layer_temperature'   ,'mixed_layer_temperature'         ,'temperature'                      ,2  ,LL   ,3  ,TYX , 8;
%   'mixed_layer_salinity'      ,'mixed_layer_salinity'            ,'salinity'                         ,2  ,LL   ,3  ,TYX , 7;
%   'mixed_layer_density'       ,'mixed_layer_density'             ,'density'                          ,2  ,LL   ,3  ,TYX , 6;
    'eastward_velocity'         ,'u'                               ,'eastward_velocity (u)'            ,3  ,LLD  ,4  ,TDYX, 4;
    'northward_velocity'        ,'v'                               ,'northward_velocity (v)'           ,3  ,LLD  ,4  ,TDYX, 5;
    'temperature'               ,'temperature'                     ,'temperature'                      ,3  ,LLD  ,4  ,TDYX, 3;
    'salinity'                  ,'salinity'                        ,'salinity'                         ,3  ,LLD  ,4  ,TDYX, 2;
%   'density'                   ,'density'                         ,'density'                          ,3  ,LLD  ,4  ,TDYX, 1;
%   'latitudeYX'                ,'Latitude'                        ,'Latitude(Y,X)'             ,2  ,LL   ,3  ,TYX ,91;
%   'longitudeYX'               ,'Longitude'                       ,'Longitude(Y,X)'            ,2  ,LL   ,3  ,TYX ,92;
     };

DSI.Fields_NameDS  =DSI.Fields(:,2);
DSI.Fields_NameMenu=DSI.Fields(:,3);
DSI.Fields_Dim     =DSI.Fields(:,4);
DSI.Fields_Coord   =DSI.Fields(:,5);
DSI.Fields_DimDS   =DSI.Fields(:,6);
DSI.Fields_CoordDS =DSI.Fields(:,7);
DSI.Fields_CBNum   =DSI.Fields(:,8);

% ml__FillValue: 1.2677e+030 for all fields
DSI.FormulaNaN='f(find(f>1.e30))=NaN;';

loaddap('+v',[URL '?MT'])
t0=datenum('1900-12-31 00:00:00','yyyy-mm-dd HH:MM:SS');
t=t0+MT;
DSI.TimeA=t;
DSI.TimeB=t+1;
DSI.Time=t+0.5;

NY=3298;NX=4500;

loaddap('+v',[URL '?Latitude[0:1:3297][0]'])
DSI.Latitude=Latitude.Latitude;
loaddap('+v',[URL '?Longitude[0][0:1:4499]'])
DSI.Longitude=Longitude.Longitude';
loaddap('+v',[URL '?Depth'])
DSI.Depth=Depth;

DSI.Range.Time1=min(DSI.TimeA);
DSI.Range.Time2=max(DSI.TimeB);
DSI.Range.Latitude1=min(DSI.Latitude);
DSI.Range.Latitude2=max(DSI.Latitude);
DSI.Range.Longitude1=min(DSI.Longitude);
DSI.Range.Longitude2=max(DSI.Longitude);

S1=['time in seconds since reference_time;'];
S2=['missing values replaced with NaN;'];
S3=['variables converted to physical units;'];
S4=['attributes have been acquired from original data.'];
DSI.Readme=[S1 '/' S2 '/' S3 '/' S4];

eval(['DSI_' DSI.DataSetBranch '=DSI;'])
if ~exist('dsi_hycom.mat','file')
eval(['save dsi_hycom.mat DSI_' DSI.DataSetBranch ';'])
end
eval(['save dsi_hycom.mat -append DSI_' DSI.DataSetBranch ';'])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function DSI=DSI_HYCOM_Global_Hindcast
% create DataSetInventory of HYCOM

% Example of loaddap argument
% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
DSI.DataSetBranch='HYCOM_Global_Hindcast';

% a general request employed by get_DataSet program has the following pattern
% loaddap([URLSITE URLPATH URLFILE '?' URLCVAR URLCTIME URLCLAT URLCLON URLCDEPTH])
%   constraints URLCTIME URLCLAT URLCLON URLCDEPTH can be in different order
%   constraints URLCLAT URLCLON URLCDEPTH will be generated by program get

% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
DSI.URLSITE='http://hycom.coaps.fsu.edu';
DSI.URLPATH='/thredds/dodsC/';
DSI.URLFILE='glb_analysis';
URL=[DSI.URLSITE DSI.URLPATH DSI.URLFILE];

% A=loaddap('-A','http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis')
%A = 
%                                   Y: [1x1 struct]
%                                   X: [1x1 struct]
%                            Latitude: [1x1 struct]
%                           Longitude: [1x1 struct]
%                                  MT: [1x1 struct]
%                                Date: [1x1 struct]
%                                qtot: [1x1 struct]
%                                 emp: [1x1 struct]
%           surface_temperature_trend: [1x1 struct]
%              surface_salinity_trend: [1x1 struct]
%                                 ssh: [1x1 struct]
%              mixed_layer_u_velocity: [1x1 struct]
%              mixed_layer_v_velocity: [1x1 struct]
%    surface_boundary_layer_thickness: [1x1 struct]
%               mixed_layer_thickness: [1x1 struct]
%             mixed_layer_temperature: [1x1 struct]
%                mixed_layer_salinity: [1x1 struct]
%                 mixed_layer_density: [1x1 struct]
%                               Depth: [1x1 struct]
%                                   u: [1x1 struct]
%                                   v: [1x1 struct]
%                         temperature: [1x1 struct]
%                            salinity: [1x1 struct]
%                   Global_Attributes: [1x1 struct]

% define list of fields or variables available in the dataset
LL={'latitude','longitude'};
LLD={'latitude','longitude','depth'};
TYX={'MT','Y','X'};
TDYX={'MT','Depth','Y','X'};

DSI.Fields={
%    opendap                      dataset                            menu                        opendap   dataset
%    name                       , name                             , name                       ,dim,coord,dim,coord,CBNum
    'downward_heat_flux_in_air' ,'qtot'                            ,'downward_heat_flux_in_air (qtot)' ,2  ,LL   ,3  ,TYX ,16;
    'water_flux_into_ocean'     ,'emp'                             ,'water_flux_into_ocean (emp)'      ,2  ,LL   ,3  ,TYX ,17;
    'surface_temperature_trend' ,'surface_temperature_trend'       ,'temperature_trend'                ,2  ,LL   ,3  ,TYX ,13;
    'surface_salinity_trend'    ,'surface_salinity_trend'          ,'salinity_trend'                   ,2  ,LL   ,3  ,TYX ,12;
    'sea_surface_elevation'     ,'ssh'                             ,'sea_surface_elevation (ssh)'      ,2  ,LL   ,3  ,TYX ,14;
    'mixed_layer_u_velocity'    ,'mixed_layer_u_velocity'          ,'u_velocity'                       ,2  ,LL   ,3  ,TYX ,10;
    'mixed_layer_v_velocity'    ,'mixed_layer_v_velocity'          ,'v_velocity'                       ,2  ,LL   ,3  ,TYX ,11;
    'boundary_layer_thickness'  ,'surface_boundary_layer_thickness','boundary_layer_thickness'         ,2  ,LL   ,3  ,TYX ,15;
    'thickness'                 ,'mixed_layer_thickness'           ,'thickness'                        ,2  ,LL   ,3  ,TYX , 9;
    'mixed_layer_temperature'   ,'mixed_layer_temperature'         ,'temperature'                      ,2  ,LL   ,3  ,TYX , 8;
    'mixed_layer_salinity'      ,'mixed_layer_salinity'            ,'salinity'                         ,2  ,LL   ,3  ,TYX , 7;
    'mixed_layer_density'       ,'mixed_layer_density'             ,'density'                          ,2  ,LL   ,3  ,TYX , 6;
    'eastward_velocity'         ,'u'                               ,'eastward_velocity (u)'            ,3  ,LLD  ,4  ,TDYX, 4;
    'northward_velocity'        ,'v'                               ,'northward_velocity (v)'           ,3  ,LLD  ,4  ,TDYX, 5;
    'temperature'               ,'temperature'                     ,'temperature'                      ,3  ,LLD  ,4  ,TDYX, 3;
    'salinity'                  ,'salinity'                        ,'salinity'                         ,3  ,LLD  ,4  ,TDYX, 2;
%   'density'                   ,'density'                         ,'density'                          ,3  ,LLD  ,4  ,TDYX, 1;
%   'latitudeYX'                ,'Latitude'                        ,'Latitude(Y,X)'             ,2  ,LL   ,3  ,TYX ,91;
%   'longitudeYX'               ,'Longitude'                       ,'Longitude(Y,X)'            ,2  ,LL   ,3  ,TYX ,92;
     };

DSI.Fields_NameDS  =DSI.Fields(:,2);
DSI.Fields_NameMenu=DSI.Fields(:,3);
DSI.Fields_Dim     =DSI.Fields(:,4);
DSI.Fields_Coord   =DSI.Fields(:,5);
DSI.Fields_DimDS   =DSI.Fields(:,6);
DSI.Fields_CoordDS =DSI.Fields(:,7);
DSI.Fields_CBNum   =DSI.Fields(:,8);

% ml__FillValue: 1.2677e+030 for all fields
DSI.FormulaNaN='f(find(f>1.e30))=NaN;';

loaddap('+v',[URL '?MT'])
t0=datenum('1900-12-31 00:00:00','yyyy-mm-dd HH:MM:SS');
t=t0+MT;
DSI.TimeA=t;
DSI.TimeB=t+1;
DSI.Time=t+0.5;

NY=3298;NX=4500;

loaddap('+v',[URL '?Latitude[0:1:3297][0]'])
DSI.Latitude=Latitude.Latitude;
loaddap('+v',[URL '?Longitude[0][0:1:4499]'])
DSI.Longitude=Longitude.Longitude';
loaddap('+v',[URL '?Depth'])
DSI.Depth=Depth;

DSI.Range.Time1=min(DSI.TimeA);
DSI.Range.Time2=max(DSI.TimeB);
DSI.Range.Latitude1=min(DSI.Latitude);
DSI.Range.Latitude2=max(DSI.Latitude);
DSI.Range.Longitude1=min(DSI.Longitude);
DSI.Range.Longitude2=max(DSI.Longitude);

S1=['time in seconds since reference_time;'];
S2=['missing values replaced with NaN;'];
S3=['variables converted to physical units;'];
S4=['attributes have been acquired from original data.'];
DSI.Readme=[S1 '/' S2 '/' S3 '/' S4];


eval(['DSI_' DSI.DataSetBranch '=DSI;'])
if ~exist('dsi_hycom.mat','file')
eval(['save dsi_hycom.mat DSI_' DSI.DataSetBranch ';'])
end
eval(['save dsi_hycom.mat -append DSI_' DSI.DataSetBranch ';'])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function DSI=DSI_HYCOM_Global_Simulation
% create DataSetInventory of HYCOM

% Example of loaddap argument
% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
DSI.DataSetBranch='HYCOM_Global_Simulation';

% a general request employed by get_DataSet program has the following pattern
% loaddap([URLSITE URLPATH URLFILE '?' URLCVAR URLCTIME URLCLAT URLCLON URLCDEPTH])
%   constraints URLCTIME URLCLAT URLCLON URLCDEPTH can be in different order
%   constraints URLCLAT URLCLON URLCDEPTH will be generated by program get

% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
DSI.URLSITE='http://hycom.coaps.fsu.edu';
DSI.URLPATH='/thredds/dodsC/';
DSI.URLFILE='glb_simulation';
URL=[DSI.URLSITE DSI.URLPATH DSI.URLFILE];

%A=loaddap('-A','http://hycom.coaps.fsu.edu/thredds/dodsC/glb_simulation')
%A = 
%                                   Y: [1x1 struct]
%                                   X: [1x1 struct]
%                            Latitude: [1x1 struct]
%                           Longitude: [1x1 struct]
%                                  MT: [1x1 struct]
%                                Date: [1x1 struct]
%                                qtot: [1x1 struct]
%                                 emp: [1x1 struct]
%           surface_temperature_trend: [1x1 struct]
%              surface_salinity_trend: [1x1 struct]
%                                 ssh: [1x1 struct]
%    surface_boundary_layer_thickness: [1x1 struct]
%               mixed_layer_thickness: [1x1 struct]
%                               Depth: [1x1 struct]
%                                   u: [1x1 struct]
%                                   v: [1x1 struct]
%                         temperature: [1x1 struct]
%                            salinity: [1x1 struct]
%                             density: [1x1 struct]
%                   Global_Attributes: [1x1 struct]

% define list of fields or variables available in the dataset
LL={'latitude','longitude'};
LLD={'latitude','longitude','depth'};
TYX={'MT','Y','X'};
TDYX={'MT','Depth','Y','X'};

DSI.Fields={
%    opendap                      dataset                            menu                        opendap   dataset
%    name                       , name                             , name                       ,dim,coord,dim,coord,CBNum
    'downward_heat_flux_in_air' ,'qtot'                            ,'downward_heat_flux_in_air (qtot)' ,2  ,LL   ,3  ,TYX ,16;
    'water_flux_into_ocean'     ,'emp'                             ,'water_flux_into_ocean (emp)'      ,2  ,LL   ,3  ,TYX ,17;
    'surface_temperature_trend' ,'surface_temperature_trend'       ,'temperature_trend'                ,2  ,LL   ,3  ,TYX ,13;
    'surface_salinity_trend'    ,'surface_salinity_trend'          ,'salinity_trend'                   ,2  ,LL   ,3  ,TYX ,12;
    'sea_surface_elevation'     ,'ssh'                             ,'sea_surface_elevation (ssh)'      ,2  ,LL   ,3  ,TYX ,14;
%   'mixed_layer_u_velocity'    ,'mixed_layer_u_velocity'          ,'u_velocity'                       ,2  ,LL   ,3  ,TYX ,10;
%   'mixed_layer_v_velocity'    ,'mixed_layer_v_velocity'          ,'v_velocity'                       ,2  ,LL   ,3  ,TYX ,11;
    'boundary_layer_thickness'  ,'surface_boundary_layer_thickness','boundary_layer_thickness'         ,2  ,LL   ,3  ,TYX ,15;
    'thickness'                 ,'mixed_layer_thickness'           ,'thickness'                        ,2  ,LL   ,3  ,TYX , 9;
%   'mixed_layer_temperature'   ,'mixed_layer_temperature'         ,'temperature'                      ,2  ,LL   ,3  ,TYX , 8;
%   'mixed_layer_salinity'      ,'mixed_layer_salinity'            ,'salinity'                         ,2  ,LL   ,3  ,TYX , 7;
%   'mixed_layer_density'       ,'mixed_layer_density'             ,'density'                          ,2  ,LL   ,3  ,TYX , 6;
    'eastward_velocity'         ,'u'                               ,'eastward_velocity (u)'            ,3  ,LLD  ,4  ,TDYX, 4;
    'northward_velocity'        ,'v'                               ,'northward_velocity (v)'           ,3  ,LLD  ,4  ,TDYX, 5;
    'temperature'               ,'temperature'                     ,'temperature'                      ,3  ,LLD  ,4  ,TDYX, 3;
    'salinity'                  ,'salinity'                        ,'salinity'                         ,3  ,LLD  ,4  ,TDYX, 2;
    'density'                   ,'density'                         ,'density'                          ,3  ,LLD  ,4  ,TDYX, 1;
%   'latitudeYX'                ,'Latitude'                        ,'Latitude(Y,X)'             ,2  ,LL   ,3  ,TYX ,91;
%   'longitudeYX'               ,'Longitude'                       ,'Longitude(Y,X)'            ,2  ,LL   ,3  ,TYX ,92;
     };

DSI.Fields_NameDS  =DSI.Fields(:,2);
DSI.Fields_NameMenu=DSI.Fields(:,3);
DSI.Fields_Dim     =DSI.Fields(:,4);
DSI.Fields_Coord   =DSI.Fields(:,5);
DSI.Fields_DimDS   =DSI.Fields(:,6);
DSI.Fields_CoordDS =DSI.Fields(:,7);
DSI.Fields_CBNum   =DSI.Fields(:,8);

% ml__FillValue: 1.2677e+030 for all fields
DSI.FormulaNaN='f(find(f>1.e30))=NaN;';

loaddap('+v',[URL '?MT'])
t0=datenum('1900-12-31 00:00:00','yyyy-mm-dd HH:MM:SS');
t=t0+MT;
DSI.TimeA=t;
DSI.TimeB=t+1;
DSI.Time=t+0.5;

NY=3298;NX=4500;

loaddap('+v',[URL '?Latitude[0:1:3297][0]'])
DSI.Latitude=Latitude.Latitude;
loaddap('+v',[URL '?Longitude[0][0:1:4499]'])
DSI.Longitude=Longitude.Longitude';
loaddap('+v',[URL '?Depth'])
DSI.Depth=Depth;

DSI.Range.Time1=min(DSI.TimeA);
DSI.Range.Time2=max(DSI.TimeB);
DSI.Range.Latitude1=min(DSI.Latitude);
DSI.Range.Latitude2=max(DSI.Latitude);
DSI.Range.Longitude1=min(DSI.Longitude);
DSI.Range.Longitude2=max(DSI.Longitude);

S1=['time in seconds since reference_time;'];
S2=['missing values replaced with NaN;'];
S3=['variables converted to physical units;'];
S4=['attributes have been acquired from original data.'];
DSI.Readme=[S1 '/' S2 '/' S3 '/' S4];


eval(['DSI_' DSI.DataSetBranch '=DSI;'])
if ~exist('dsi_hycom.mat','file')
eval(['save dsi_hycom.mat DSI_' DSI.DataSetBranch ';'])
end
eval(['save dsi_hycom.mat -append DSI_' DSI.DataSetBranch ';'])
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function DSI=DSI_HYCOM_Atlantic_Ocean_Prediction_System
% create DataSetInventory of HYCOM

% Example of loaddap argument
% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
DSI.DataSetBranch='HYCOM_Atlantic_Ocean_Prediction_System';

% a general request employed by get_DataSet program has the following pattern
% loaddap([URLSITE URLPATH URLFILE '?' URLCVAR URLCTIME URLCLAT URLCLON URLCDEPTH])
%   constraints URLCTIME URLCLAT URLCLON URLCDEPTH can be in different order
%   constraints URLCLAT URLCLON URLCDEPTH will be generated by program get

% http://hycom.coaps.fsu.edu/thredds/dodsC/glb_analysis?temperature[0:1:1][0:1:2][0:1:3][0:1:4]
DSI.URLSITE='http://hycom.coaps.fsu.edu';
DSI.URLPATH='/thredds/dodsC/';
DSI.URLFILE='atl_ops';
URL=[DSI.URLSITE DSI.URLPATH DSI.URLFILE];

%A=loaddap('-A','http://hycom.coaps.fsu.edu/thredds/dodsC/atl_ops')
%A = 
%                            Latitude: [1x1 struct]
%                           Longitude: [1x1 struct]
%                                  MT: [1x1 struct]
%                                Date: [1x1 struct]
%                                qtot: [1x1 struct]
%                                 emp: [1x1 struct]
%           surface_temperature_trend: [1x1 struct]
%              surface_salinity_trend: [1x1 struct]
%                                 ssh: [1x1 struct]
%              mixed_layer_u_velocity: [1x1 struct]
%              mixed_layer_v_velocity: [1x1 struct]
%    surface_boundary_layer_thickness: [1x1 struct]
%               mixed_layer_thickness: [1x1 struct]
%             mixed_layer_temperature: [1x1 struct]
%                mixed_layer_salinity: [1x1 struct]
%                 mixed_layer_density: [1x1 struct]
%                               Depth: [1x1 struct]
%                                   u: [1x1 struct]
%                                   v: [1x1 struct]
%                         temperature: [1x1 struct]
%                            salinity: [1x1 struct]
%                             density: [1x1 struct]
%                   Global_Attributes: [1x1 struct]

% define list of fields or variables available in the dataset
LL={'latitude','longitude'};
LLD={'latitude','longitude','depth'};
TYX={'MT','Y','X'};
TDYX={'MT','Depth','Y','X'};

DSI.Fields={
%    opendap                      dataset                            menu                        opendap   dataset
%    name                       , name                             , name                       ,dim,coord,dim,coord,CBNum
    'downward_heat_flux_in_air' ,'qtot'                            ,'downward_heat_flux_in_air (qtot)' ,2  ,LL   ,3  ,TYX ,16;
    'water_flux_into_ocean'     ,'emp'                             ,'water_flux_into_ocean (emp)'      ,2  ,LL   ,3  ,TYX ,17;
    'surface_temperature_trend' ,'surface_temperature_trend'       ,'temperature_trend'                ,2  ,LL   ,3  ,TYX ,13;
    'surface_salinity_trend'    ,'surface_salinity_trend'          ,'salinity_trend'                   ,2  ,LL   ,3  ,TYX ,12;
    'sea_surface_elevation'     ,'ssh'                             ,'sea_surface_elevation (ssh)'      ,2  ,LL   ,3  ,TYX ,14;
    'mixed_layer_u_velocity'    ,'mixed_layer_u_velocity'          ,'u_velocity'                       ,2  ,LL   ,3  ,TYX ,10;
    'mixed_layer_v_velocity'    ,'mixed_layer_v_velocity'          ,'v_velocity'                       ,2  ,LL   ,3  ,TYX ,11;
    'boundary_layer_thickness'  ,'surface_boundary_layer_thickness','boundary_layer_thickness'         ,2  ,LL   ,3  ,TYX ,15;
    'thickness'                 ,'mixed_layer_thickness'           ,'thickness'                        ,2  ,LL   ,3  ,TYX , 9;
    'mixed_layer_temperature'   ,'mixed_layer_temperature'         ,'temperature'                      ,2  ,LL   ,3  ,TYX , 8;
    'mixed_layer_salinity'      ,'mixed_layer_salinity'            ,'salinity'                         ,2  ,LL   ,3  ,TYX , 7;
    'mixed_layer_density'       ,'mixed_layer_density'             ,'density'                          ,2  ,LL   ,3  ,TYX , 6;
    'eastward_velocity'         ,'u'                               ,'eastward_velocity (u)'            ,3  ,LLD  ,4  ,TDYX, 4;
    'northward_velocity'        ,'v'                               ,'northward_velocity (v)'           ,3  ,LLD  ,4  ,TDYX, 5;
    'temperature'               ,'temperature'                     ,'temperature'                      ,3  ,LLD  ,4  ,TDYX, 3;
    'salinity'                  ,'salinity'                        ,'salinity'                         ,3  ,LLD  ,4  ,TDYX, 2;
    'density'                   ,'density'                         ,'density'                          ,3  ,LLD  ,4  ,TDYX, 1;
    %'latitudeYX'                ,'Latitude'                        ,'Latitude(Y,X)'             ,2  ,LL   ,3  ,TYX ,91;
    %'longitudeYX'               ,'Longitude'                       ,'Longitude(Y,X)'            ,2  ,LL   ,3  ,TYX ,92;
     };

DSI.Fields_NameDS  =DSI.Fields(:,2);
DSI.Fields_NameMenu=DSI.Fields(:,3);
DSI.Fields_Dim     =DSI.Fields(:,4);
DSI.Fields_Coord   =DSI.Fields(:,5);
DSI.Fields_DimDS   =DSI.Fields(:,6);
DSI.Fields_CoordDS =DSI.Fields(:,7);
DSI.Fields_CBNum   =DSI.Fields(:,8);

% ml__FillValue: 1.2677e+030 for all fields
DSI.FormulaNaN='f(find(f>1.e30))=NaN;';

loaddap('+v',[URL '?MT'])
t0=datenum('1900-12-31 00:00:00','yyyy-mm-dd HH:MM:SS');
t=t0+MT;
DSI.TimeA=t;
DSI.TimeB=t+1;
DSI.Time=t+0.5;

NY=3298;NX=4500;

% in this DataSet Latitude/Longitude is 1D array
loaddap('+v',[URL '?Latitude'])
DSI.Latitude=Latitude;
loaddap('+v',[URL '?Longitude'])
DSI.Longitude=Longitude;
loaddap('+v',[URL '?Depth'])
DSI.Depth=Depth;

DSI.Range.Time1=min(DSI.TimeA);
DSI.Range.Time2=max(DSI.TimeB);
DSI.Range.Latitude1=min(DSI.Latitude);
DSI.Range.Latitude2=max(DSI.Latitude);
DSI.Range.Longitude1=min(DSI.Longitude);
DSI.Range.Longitude2=max(DSI.Longitude);

S1=['time in seconds since reference_time;'];
S2=['missing values replaced with NaN;'];
S3=['variables converted to physical units;'];
S4=['attributes have been acquired from original data.'];
DSI.Readme=[S1 '/' S2 '/' S3 '/' S4];

eval(['DSI_' DSI.DataSetBranch '=DSI;'])
if ~exist('dsi_hycom.mat','file')
eval(['save dsi_hycom.mat DSI_' DSI.DataSetBranch ';'])
end
eval(['save dsi_hycom.mat -append DSI_' DSI.DataSetBranch ';'])


